@@ -1,11 +1,9 @@
 package Thrift::SASL::Transport;
-
+$Thrift::SASL::Transport::VERSION = '0.006';
 use strict;
 use warnings;
 use Data::Dumper;
 
-our $VERSION = '0.005'; # VERSION
-
 # Nasty hack to make the Thrift libs handle the extra 4-bytes
 # header put by GSSAPI in front of unencoded (auth only) replies
 
@@ -110,11 +108,23 @@ sub _sasl_handshake {
             #$client->client_step($data // '') if $client->need_step;
             last;
         }
+
+        my $extra_msg = $self->__probe_env_and_xs_sasl_bug( $client );
+
         if ( $code == SASL_BAD || $code == SASL_ERROR ) {
-            die "Authentication failed: $code > $data";
+            die sprintf "Authentication failed: %s > %s%s",
+                            $code,
+                            $data,
+                            $extra_msg ? '. ' . $extra_msg : '',
+            ;
         }
+
         $resp = $client->client_step($data);
-        die "Client rejected authentication" unless defined $data;
+        if ( ! defined $data ) {
+            die sprintf 'Client rejected authentication%s',
+                            $extra_msg ? '. ' . $extra_msg : '',
+            ;
+        }
     }
 
     $self->{_sasl_encode_check} = 1;
@@ -123,6 +133,26 @@ sub _sasl_handshake {
     return $self->{_sasl_client};
 }
 
+sub __probe_env_and_xs_sasl_bug {
+    # See: https://github.com/Perl-Hadoop/Thrift-SASL/issues/1
+    #
+    my($self, $client) = @_;
+    return if      ! $client->isa('Authen::SASL::XS')
+                && ! $client->isa('Authen::SASL::Cyrus')
+    ;
+
+    return if exists $ENV{USER} || exists $ENV{USERNAME};
+
+    my $sasl_class = ref $client;
+
+    return join ' ',
+        "cyrus-sasl and in turn $sasl_class needs either USER or USERNAME",
+        'environment variable to be present and they seem to be missing',
+        'in your environment.',
+        'The error you have received might be caused by that.',
+    ;
+}
+
 sub write {
     my $self   = shift;
     my $buffer = shift;
@@ -234,7 +264,7 @@ Thrift::SASL::Transport - Thrift Transport allowing Kerberos auth/encryption thr
 
 =head1 VERSION
 
-version 0.005
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -275,6 +305,24 @@ Initial version with simple SASL authentication (LDAP) developped by Vikentiy Fe
 
 Thanks to my employer Booking.com to allow me to release this module for public use
 
+=for Pod::Coverage BinaryProtocolOverride_readMessageBegin
+
+=for Pod::Coverage close
+
+=for Pod::Coverage flush
+
+=for Pod::Coverage isOpen
+
+=for Pod::Coverage new
+
+=for Pod::Coverage open
+
+=for Pod::Coverage read
+
+=for Pod::Coverage readAll
+
+=for Pod::Coverage write
+
 =head1 AUTHOR
 
 David Morel <david.morel@amakuru.net>
